function [prest,prvar] =ProjDirectMV(x,Gi)
% projects time series missing values onto observed for m by n series x
% Uses inverse covariance matrix Gi of x taken by columns of x giving blocks of size mxm 
% Solves equations directly for the missing values in terms of know series values
% prest is the original series with missing values replaced by projected
% prvar has same with zeros except variances in place of missing values 
%
% disp('Release version of ProjDirectMV')
%
[m,n]=size(x); % series dimension and length
[d,dd]=size(Gi); 
if (d~=m*n)|(d~=dd)
    disp('non-conformable parameters')
    return
end
prest=x;
prvar=zeros(m,n);
nmv=sum(sum(isnan(x)));
A=zeros(nmv);
B=zeros(nmv,1);
ct1=0;
u=0;
% Set up equations to be solved for missing values
for t=1:n
    for i=1:m
        u=u+1;
        if isnan(x(i,t))
            ct1=ct1+1;
            ct2=0;
            rsum=0;
            v=0;
            for s=1:n
                for j=1:m
                    v=v+1;
                    if isnan(x(j,s))
                        ct2=ct2+1;
                        A(ct1,ct2)=Gi(u,v);
                    else
                        rsum=rsum+Gi(u,v)*x(j,s);
                    end
                end
            end
            B(ct1)=-rsum;
        end
    end
end
% Solve the equations and restructure the missing values and their variances
C=A\B; 
D=diag(inv(A));
ct1=0;
for t=1:n
    for i=1:m
        u=u+1;
        if isnan(x(i,t))
            ct1=ct1+1;
            prest(i,t)=C(ct1);
            prvar(i,t)=D(ct1);
        end
    end
end

return


