function [prest,prvar,Lincoef] =ProjLinCoefMV(x,G)
% projects time series missing values onto observed for m by n series x
% Uses covariance matrix G of x taken by columns of x giving blocks of size mxm 
% Solves for linear coefficients of projection from which missing values are derived
% prest is the original series with missing values replaced by projected values
% prvar has same with zeros except variances in place of missing values 
% Lincoeff is same size as G with coeffs of missing values in terms of known, elsewhere zeros
%
% disp('Release version of ProjLinCoefMV')
%
[m,n]=size(x); % series dimension and length
[d,dd]=size(G); 
if (d~=m*n)|(d~=dd)
    disp('non-conformable parameters')
    return
end
prest=x;
prvar=zeros(m,n);
nmv=sum(sum(isnan(x))); % number missing vlaues
nkv=d-nmv;  % number known values
A=zeros(nkv);
B=zeros(nmv,nkv);
C=zeros(nmv);
Lincoef=zeros(d);
ct1=0;
ct4=0;
u=0;
% Set up equations for the linear projection coefficients
for t=1:n
    for i=1:m
        u=u+1;
        if ~isnan(x(i,t))
            ct1=ct1+1;
            ct2=0;
            ct3=0;
            v=0;
            for s=1:n
                for j=1:m
                    v=v+1;
                    if ~isnan(x(j,s))
                        ct2=ct2+1;
                        A(ct1,ct2)=G(u,v);
                    else
                        ct3=ct3+1;
                        B(ct3,ct1)=G(u,v);
                    end
                end
            end
        else
            v=0;
            ct3=0;
            ct4=ct4+1;
            for s=1:n
                for j=1:m
                    v=v+1;
                    if isnan(x(j,s))
                        ct3=ct3+1;
                        C(ct4,ct3)=G(u,v);
                    end
                end
            end
        end 
    end
end
L=B/A; % Solve for the linear projection coefficients vector
D=diag(C-B*L');
% copy linear coefficients to matrix structure
ct1=0;
u=0;
for t=1:n
    for i=1:m
        u=u+1;
        if isnan(x(i,t))
            ct1=ct1+1;
            prvar(i,t)=D(ct1);
            ct3=0;
            v=0;
            for s=1:n
                for j=1:m
                    v=v+1;
                    if ~isnan(x(j,s))
                        ct3=ct3+1;
                        Lincoef(u,v)=L(ct1,ct3);
                    end
                end
            end
        end
    end
end
% size(Lincoef)
X=reshape(x,d,1);
% size(X)
I=find(isnan(X));
X(I)=0;
% construct the predictions vector and copy into missing series values
X=X+Lincoef*X;
prest=reshape(X,m,n);


return


