function G=VARcovfun(Phi,SigPhi,K)
% forms the covariance function G up to lag K of a VAR model 
% with coefficients Phi and innovation variance SigPhi
%
% Method: the numerator of the covariance generating function (CGF) is determined
% then the CGF is expanded.
%
% G(:,:,1) is the lag zero covariance matrix or series variance
% G(:,:,k+1) is the lag k covariance matrix
%
% disp('Release version of VARcovfun')
[m,mm,p]=size(Phi);
if nargin==2
    K=p;
end
phi=zeros(m,m,p+1);
phi(:,:,1)=eye(m);
for k=1:p
    phi(:,:,k+1)=-squeeze(Phi(:,:,k));
end
N=m*m*p+m*(m+1)/2; % number of equations to be solved for numerator of CGF
A=zeros(N,N);
b=zeros(N,1);
sol=b;
% set up the equations to be solved
ct1=0;
for k1=0:p
    for i1=1:m
        if k1>0
            j1lo=1;
        else
            j1lo=i1;
        end
        for j1=j1lo:m
            ct1=ct1+1;
            ct2=0;
            if (k1==0)
                b(ct1)=SigPhi(i1,j1);
            end
            for k2=0:p
                for i2=1:m
                    if k2>0
                        j2lo=1;
                    else
                        j2lo=i2;
                    end
                    for j2=j2lo:m
                        ct2=ct2+1;
                        term=0;
                        if (i1==i2)&(k2>=k1)
                            term=term+phi(j1,j2,k2-k1+1);
                        end
                        if (j1==i2)&(k2+k1<=p)
                            term=term+phi(i1,j2,k2+k1+1);
                        end
                        A(ct1,ct2)=term;
                    end
                end
            end
        end
    end
end
sol=A\b; % vector solution for numerator coefficients
del=zeros(m,m,p+1);
% matrix coefficients reconstructed from vector solution
ct1=0;
for k1=0:p
    for i1=1:m
        if k1>0
            j1lo=1;
        else
            j1lo=i1;
        end
        for j1=j1lo:m
            ct1=ct1+1;
            del(i1,j1,k1+1)=sol(ct1);
        end
    end
end
% Expansion of the rational matrix function to give the CGF
G=zeros(m,m,K+1);
G(:,:,1)=del(:,:,1);
for k=1:K
    if k<=p
        H=del(:,:,k+1);
    else
        H=0;
    end
    for u=1:min(k,p)
        H=H+Phi(:,:,u)*G(:,:,k+1-u);
    end
    G(:,:,k+1)=H;
end
G(:,:,1)=G(:,:,1)+G(:,:,1)';

return



        