function G=VARcovfunX(Phi,SigPhi,K)
% forms the covariance function G up to lag K of a VAR model 
% with coefficients Phi and innovation variance SigPhi
%
% The Yule Walker equations are re-arranged and solved
%
% G(:,:,1) is the lag zero covariance matrix or series variance
% G(:,:,k+1) is the lag k covariance matrix
%
% disp('Release version of VARcovfunX')
[m,mm,p]=size(Phi);
if nargin==2
    K=p;
end
phi=zeros(m,m,p+1);
phi(:,:,1)=eye(m);
for k=1:p
    phi(:,:,k+1)=-squeeze(Phi(:,:,k));
end
N=m*m*p+m*(m+1)/2; % number of equations to be solved for numerator of CGF
A=zeros(N,N);
b=zeros(N,1);
sol=b;
% set up the equations to be solved
ct1=0;
for k1=0:p
    for i1=1:m
        if k1>0
            j1lo=1;
        else
            j1lo=i1;
        end
        for j1=j1lo:m
            ct1=ct1+1;
            ct2=0;
            if (k1==0)
                b(ct1)=SigPhi(i1,j1);
            end
            for k2=0:p
                for i2=1:m
                    if k2>0
                        j2lo=1;
                    else
                        j2lo=i2;
                    end
                    for j2=j2lo:m
                        ct2=ct2+1;
                        term=0;
                        if (i2==j1)&(k1+k2<=p)
                            term=term+phi(i1,j2,k1+k2+1);
                        end
                        if (j1==j2)&(k1>=k2)
                            term=term+phi(i1,i2,k1-k2+1);
                        end
                        A(ct1,ct2)=term;
                    end
                end
            end
        end
    end
end
% A
% b
sol=A\b; % vector solution for numerator coefficients
G=zeros(m,m,K+1);
% matrix coefficients reconstructed from vector solution
ct1=0;
for k1=0:p
    for i1=1:m
        if k1>0
            j1lo=1;
        else
            j1lo=i1;
        end
        for j1=j1lo:m
            ct1=ct1+1;
            G(i1,j1,k1+1)=sol(ct1);
        end
    end
end
% Extension of covariances to higher lags
for k=p+1:K
    H=0;
    for u=1:p
        H=H+Phi(:,:,u)*G(:,:,k+1-u);
    end
    G(:,:,k+1)=H;
end
G(:,:,1)=G(:,:,1)+G(:,:,1)';

return



        