function [W]=VARinvcovmat(Phi,SigPhi,K)
% forms the inverse covariance matrix W of the stacked states up to order K of a VAR(p) model
% The method is based on formula (2.61) inthe book, using the forward and a backward models 
% The backward model is derived from the forward model by factorising its transposed covariance function
% disp('Release version of VARinvcovmat')
[m,mm,p]=size(Phi);
if nargin==2
    K=p;
end
if K<(p-1)
    disp('order too low')
    return
end
phist=zeros(m,m,p+1); % structural form with unit uncorrelated innovations
Phi0=inv(chol(SigPhi))';
phist(:,:,1)=Phi0;
for k=1:p
    phist(:,:,k+1)=-Phi0*squeeze(Phi(:,:,k));
end
% construct backward model factor 
for k=0:p
    tphist(:,:,k+1)=squeeze(phist(:,:,k+1))';
end
Ct=covfun(tphist,m,p);
for k=0:p
    C(:,:,k+1)=squeeze(Ct(:,:,k+1))'; % transpose for time reverse
end
maxiter=15;
tol=1.e-15*trace(squeeze(C(:,:,1)));
[thetat,eps]=covfunfactor(C,m,p,maxiter,tol);  % factorize for transpose of backward structural form
for k=0:p
    thetast(:,:,k+1)=squeeze(thetat(:,:,k+1))'; 
end
eps;
precn=max(max(max(abs(C-covfun(thetat,m,p)))));
% Construct inverse block covariance matrix of x_0,...,x_K
M=zeros(m*(K+1));
sub=[1:m];
for r=0:K
    j=min(K-r,p);
    for i=0:j
        M(r*m+sub,(r+i)*m+sub)=phist(:,:,i+1);
    end
end
L=zeros(m*p,m*(K+1));
for r=1:p
    for i=r:p
        L((r-1)*m+sub,m*(K+r-i)+sub)=thetast(:,:,i+1);
    end
end
W=M'*M-L'*L;
return