function [phi,eps]=covfunfactor(C,m,p,maxiter,tol)
% given the coefficients of a finite mxm covariance function of order p solves for the
% coefficients of the mxm polynomial factor phi from the equn. phi(z)*phi(z^-1)' = C(z)
% using an iterative scheme. 
% The maximum no. of iterations (maxiter) or the required accuracy of the solution (tol) 
% can optionally be specified else default values are used.
% The solution accuracy is described by the maximum equation discrepancy which is returned as eps
% For uniqueness of solution the coefficient phi_0 is constrained to be upper triangular
%
% disp('Release version of covfunfactor')
if nargin<4
    maxiter=10;
end
if nargin<5
    tol=1.0E-10;
end
% initialize phi
phi=zeros(m,m,p+1);
C0=squeeze(C(:,:,1));
tol=tol*trace(C0);
% get UPPER triangular R with R*R'=C0
R=flipud(fliplr(chol(flipud(fliplr(C0)))))';
phi(:,:,1)=R;
for k=2:(p+1)
    phi(:,:,k)=squeeze(C(:,:,k))/R';
end
N=m*m*p+m*(m+1)/2; % number of equations
del=zeros(N,1);
sol=del;
A=zeros(N,N);
for iter=1:maxiter
    D=C-covfunform(phi,m,p);
    eps=max(max(max(abs(D))));
    if eps<tol
        break
    end
    ct1=0;
    for k1=0:p
        for i1=1:m
            if k1>0
                j1lo=1;
            else
                j1lo=i1;
            end
            for j1=j1lo:m
                ct1=ct1+1;
                ct2=0;
                del(ct1)=D(i1,j1,k1+1);
                for k2=0:p
                    for i2=1:m
                        if k2>0
                            j2lo=1;
                        else
                            j2lo=i2;
                        end
                        for j2=j2lo:m
                            ct2=ct2+1;
                            term=0;
                            if (i1==i2)&(k2>=k1)
                                term=term+phi(j1,j2,k2-k1+1);
                            end
                            if (i2==j1)&(k2+k1<=p)
                                term=term+phi(i1,j2,k2+k1+1);
                            end
                            A(ct1,ct2)=term;
                        end
                    end
                end
            end
        end
    end
    sol=A\del;
    ct1=0;
    for k1=0:p
        for i1=1:m
            if k1>0
                j1lo=1;
            else
                j1lo=i1;
            end
            for j1=j1lo:m
                ct1=ct1+1;
                phi(i1,j1,k1+1)=phi(i1,j1,k1+1)+sol(ct1);
            end
        end
    end
end
% end of iterative loop

        

